#!/bin/bash

# Defaults - can be overridden by source'd script
PHP_FASTCGI_PASS=unix:/var/run/php5-fpm.sock
SAVEDIR=/etc/ssl/nginx/

# @param $1    Bind IP/port
# @param $2    Server name
# @param $3    Optional: Bind IP/port (HTTPS)
# @param $4    Optional: Path for certificate + chain
# @param $5    Optional: Path for keyfile
# @param stdin Config body
vhost() {
	cat <<EOD
# Server: $2
server {
  listen $1 ;
  server_name $2 ;
EOD
	if [ ! -z "$3" ] ; then
		cat <<EOD
  listen $3 ssl ;
  ssl_certificate $4 ;
  ssl_certificate_key $5 ;
EOD
	fi
	cat
	echo "}"
	
}

# @param $1    Bind IP/port
# @param $2    Server name
# @param $3    Path for certificate + chain
# @param $4    Path for keyfile
# @param stdin Config body
vhost_sslonly() {
	cat <<EOD
# Server: $2
server {
  listen $1 ;
  server_name $2 ;
  ssl_certificate $3 ;
  ssl_certificate_key $4 ;
EOD
	cat
	echo "}"
}

# @param stdin   File body
# @return stdout Path to saved file
pathfor() {

	if [ ! -d "$SAVEDIR" ] ; then
		mkdir -p "$SAVEDIR"
	fi

	local DEST=$(mktemp)
	cat > DEST
	local HASH=$(md5sum "$DEST" | cut -d' ' -f1)
	
	if [ -f "${SAVEDIR}/${HASH}" ] ; then
		# Already exists
		rm "$DEST"
		
	else
		# Save
		mv "$DEST" "${SAVEDIR}/${HASH}"
		# FIXME: Permissions?
	fi
	
	echo "${SAVEDIR}/${HASH}"
}

#
enablephp() {
	cat <<EOD
    # PHP
    location ~ \\.php\$ {
      try_files \$uri =404 ;
      fastcgi_pass $PHP_FASTCGI_PASS ;
      fastcgi_index index.php ;
      fastcgi_param SCRIPT_FILENAME \$document_root\$fastcgi_script_name ;
      include fastcgi_params ;
    }
EOD

}

# @param $1 path
# @param stdin body
location() {
	echo "  location $1 { "
	cat
	echo "  }"
}

######################
## BASIC HOST TYPES ##
######################

#
dropconnection() {
	echo "  return 444 ;"
}

# @param $1 destination
redirect() {
	echo "rewrite ^/(.*)\$ ${1}/\$1 redirect ;"
}

# @param $1 root
staticonly() {
	cat <<EOD
  root $1 ;
  location / {
    index index.html index.htm ;
  }
EOD
}

# @param $1 root
standardphp() {
	
	cat <<EOD
  root $1 ;
  location / {
    index index.php index.html index.htm ;
EOD
	enablephp
	echo "  }"
    
}

# @param $1 root
# @param $2 rewrite_dest
rewritephp() {
	cat <<EOD
  root $1
  index index.php index.html index.htm ;
EOD
	enablephp
	cat <<EOD
  location / {
    try_files \$uri $2 ;
  }
EOD

}

###############
## MODIFIERS ##
###############

# @param $1 passfile
# @param $2 name
basicauth() {
	cat <<EOD
    # HTTP Auth
    auth_basic "$2" ;
    auth_basic_user_file $1 ;
EOD
}

# @param $1 origin
allowcors() {
	cat <<EOD
  # CORS
  add_header "Access-Control-Allow-Origin" "$1" ;
  add_header "Access-Control-Allow-Credentials" "true" ;
  if ( \$request_method = "OPTIONS" ) { 
    add_header "Access-Control-Allow-Headers" "Authorization,DNT,Keep-Alive,User-Agent,Cache-Control,Content-Type" ;
    add_header "Content-Type" "text/html" ;
    add_header "Content-Length" "0" ;
    return 204 ;
  }
EOD
}

#
munincgi() {

	cat <<EOD
  location ^~ /munin-cgi/munin-cgi-graph/ {
    fastcgi_split_path_info ^(/munin-cgi/munin-cgi-graph)(.*);
    fastcgi_param PATH_INFO $fastcgi_path_info;
    fastcgi_pass unix:/var/run/munin/munin-fastcgi-graph.sock;
    include fastcgi_params;
  }
EOD

}

# @param $1 location
downloaddir() {
	cat <<EOD | location "$1"
    autoindex on ;
    autoindex_exact_size off ;
    include mime.types ;
    default_type application/octet-stream ;
EOD
}

# @param $1 upstream
passthrough() {

	cat <<EOD
    proxy_pass $1 ;
    proxy_http_version 1.1 ;
    proxy_set_header Upgrade $http_upgrade ;
    proxy_set_header Connection "upgrade" ;
EOD

}

commonheader() {
	echo -n "# Generated: " ; date
	echo "server_tokens off;"
}

commonfooter() {
	# nothing
	echo -n ""
}

#############
## CONTROL ##
#############

reindent() {
	sed 's/\t/ / ; s/^\s*//' | awk 'BEGIN { indent=0 } ; /}/{ indent -= 1 } ; { for(c=0;c<indent;c++) printf "   "; print; } ; /{/{ indent += 1 } ; '
}

usage() {
	echo "Usage: ${SCRIPT_NAME} <scriptfile.sh>" >&2
	exit 1
}

function_exists() {
	( type "$1" 2>&1 ) | grep -q "function"
}

main() {
	if [ ! -f "$1" ] ; then
		usage
	fi
	
	source "$1"
		
	if ! function_exists buildall ; then
		echo "FATAL: Sourced script does not provide buildall()" >&2
		exit 1
	fi
	
	# Ready to go
	
	(
		commonheader
		buildall
		commonfooter
	) | reindent
	
	# Done
}

SCRIPT_NAME="$0"
main "$@"
